
--[[ Use Addon Wide Data Table ]]--
local addonName,addonData = ...

--[[ Localise Addon Wide Data Table Sub Tables ]]--
local Translate = addonData.Translate

--[[ Make sure we have nUI Tables ]]--
if not nUI_InfoPanels then nUI_InfoPanels = {}; end

--[[ Allocate an ID for the infopanel ]]--
nUI_INFOMODE_WHISPERS  = 21;
nUI_INFOPANEL_WHISPERS = "nUI_InfoPanel_Whispers";

--[[ Fill in the data as required ]]--
nUI_InfoPanels[nUI_INFOPANEL_WHISPERS] =
{	
	enabled   = true,
	desc      = Translate["INFO_PANEL_TEXT"],			-- player friendly name/description of the panel
	label     = Translate["INFO_PANEL_LABEL"],	        -- label to use on the panel selection button face
	rotation  = nUI_INFOMODE_WHISPERS,					-- index or position this panel appears on/in when clicking the selector button
	full_size = true,									-- this plugin requires the entire info panel port without the button bag	
	options  =
	{
		enabled  = true,
		btn_size = 45,
		btn_gap  = -8,
	},
};

addonData.windowName = nUI_InfoPanels[nUI_INFOPANEL_WHISPERS].label;


--[[ Create the Chat Frame Plugin ]]--
local plugin            = CreateFrame( "Frame", nUI_INFOPANEL_WHISPERS, nUI_Dashboard.Anchor );
plugin.active           = true;
plugin.IPInitialised	= false;


--[[ This is called automatically by nUI to initialise the panel ]]--
plugin.initPanel = function( container, options )	
	plugin.container = container;
	plugin.options   = options;
	
	if ( not addonData.ChatFrame ) then return end

	if options and options.enabled then			
		container.texture = container:CreateTexture( "$parentBackdrop", "BACKGROUND" );		
		container.texture:SetAllPoints( container );		
		plugin.setEnabled( true );		
	end
	plugin.IPInitialised = true;
end

--[[ This is called automatically by nUI to resize the contents of the panel ]]--
plugin.sizeChanged = function( scale, height, width )
		
	local options  = plugin.options;
	local btn_size = options.btn_size * scale;

	plugin.scale = scale;	
	plugin.height = height;
	plugin.width = width;
		
	-- If we haven't created the frame yet then no need to proceed quite yet
	if ( not addonData.ChatFrame ) then return end
		
	-- Set the size of the chat frame to fit the width and height of the panel
	addonData.ChatFrame:SetWidth( width );
	addonData.ChatFrame:SetHeight( height );
	
	-- set the size of the scroll buttons in the chat frames	
	if plugin.btn_size ~= btn_size then
		plugin.btn_size = btn_size;
		addonData.BottomButton:SetWidth( btn_size );
		addonData.BottomButton:SetHeight( btn_size );		
	end
	
	-- set the locations of the buttons relative to their chat frame	
	addonData.BottomButton:ClearAllPoints();
	addonData.BottomButton:SetPoint( "BOTTOMLEFT", addonData.ChatFrame, "BOTTOMLEFT", -3 * scale, -8 * scale );	
end	

--[[ This is called automatically by nUI and manually by this addon to enable the panel ]]--
plugin.setEnabled = function( enabled )
	if plugin.enabled ~= enabled then		
		plugin.enabled = enabled;
		if not enabled then
            plugin = addonData:DisablePlugin(plugin);
			-- This little monster makes sure that we restore the chat settings to the general chat frame and closes the frame
			FCF_PopInWindow(addonData.ChatFrame);		
		else
			plugin = addonData:EnablePlugin(plugin,addonData.windowName);
			
			-- Then Add the ones we want to track
			ChatFrame_AddMessageGroup(addonData.ChatFrame, "WHISPER");
			--ChatFrame_AddMessageGroup(addonData.ChatFrame, "WHISPER_INFORM");
			ChatFrame_AddMessageGroup(addonData.ChatFrame, "BN_WHISPER");
		end				
	end			
end

--[[ This is called automatically by nUI when you select the panel ]]--
plugin.setSelected = function( selected )
	if selected ~= plugin.selected then
		plugin.selected = selected;				
		if selected then			

			-- Reset the mouseover timer
			local mouseover_timer = 0;
			
			-- Set Up the OnUpdate routine for this panel to handle mouse over updates
			local function onUpdate( who, elapsed )			
			     mouseover_timer = mouseover_timer + elapsed;				
				 
				 -- Every 5 frames per second show the buttons ( if the frame exists ) and reset the timer
				 -- Also restore the name we want in the frame just in case Blizz overwrote it
			     if mouseover_timer > 0.2 then -- 5 fps is plenty fast enough to show these buttons				
			     	mouseover_timer = 0;				
				    if ( addonData.ChatFrame ) then addonData.ChatFrame.showButtons( MouseIsOver( addonData.ChatFrame ) );	 end
					addonData:SetWindowName(addonData.windowName)
			     end
            end                						
            plugin:SetScript( "OnUpdate", onUpdate );
			
        end		
	else	
		-- Turn off the Update routine when not selected
		plugin:SetScript( "OnUpdate", nil );			
	end
end

--[[ This should no longer be necessary but just in case it allows /nuiwc disable and enable options ]]--
local function nUI_InfoPanel_Whispers_SlashCommands(msg)
    msg = string.lower(msg);
    local args = {};
    for word in string.gmatch(msg,"[^%s]+") do
        table.insert(args,word);
    end
	
	-- Disable the plugin
    if (args[1] == "disable") then 
        plugin.setEnabled(false);
		
	-- Enable the plugin
    elseif (args[1] == "enable") then
        plugin.setEnabled(true);
		
    end
	
end

--[[ Addon's OnEvent function ]]--
local function nUI_InfoPanel_Whispers_OnEvent(self,event,arg1,arg2,arg3)

	-- If we are using simple chat then we need to deactivate the addon
	if (event == "PLAYER_LOGIN") then
		if (SIMPLE_CHAT == "1") then
			DEFAULT_CHAT_FRAME:AddMessage( Translate["SIMPLE_CHAT"], 1, 0.83, 0 );
			plugin.active = false;
		end
		plugin.IPLoggedIn		= true;
		
	-- If blizz is updating the chat windows then we want in so we can do our stuff
	elseif ( event == "UPDATE_CHAT_WINDOWS" ) then

		-- Get the current list of frames
		local shownList,dockedList,infoPanelList = addonData:GetActiveFrames();
		
		local inUse = 0;
		
		-- Go through the list of available chat frames
		for i = 1, NUM_CHAT_WINDOWS do
		
			-- Generate a chat frame object
			local cf = _G["ChatFrame"..i];
			
			-- If it is not in the shown or docked list then it *should* be available for use
			if ( not shownList[cf:GetID()] and not dockedList[cf:GetID()] and not infoPanelList[cf:GetID()] ) then
			
				-- Set our special variable with the frame we're gonna use
				addonData.ChatFrame = _G["ChatFrame"..i];

				-- Tell us that we are using it in an infoPanel as that isn't docked or shown
				addonData.ChatFrame.infoPanel = true;
				
				-- Initialise the rest of its settings
				addonData:InitialiseFrame(addonData.windowName);
				
				-- We made our frame so we can stop looking for a spot
				break;
			else
				inUse = inUse + 1;
			end
		end
		
		-- Aww for shame, can't find a frame to use.
		-- If this is the first attempt to create a chat frame to use and one isn't available
		-- then display a message saying so.  Once this addon finds a free chat frame it will always use it
		-- until the WTF file is emptied
		if ( not addonData.ChatFrame ) then
			DEFAULT_CHAT_FRAME:AddMessage(format(Translate["INUSE_FORMAT"],addonName,inUse), 1, 0.83, 0 );
			plugin.active = false;
			if ( plugin.IPLoggedIn ) then 
				plugin.setEnabled(false);
			end
		else			
			plugin.active = true;
			if ( plugin.IPLoggedIn ) then 
				plugin.initPanel( plugin.container, plugin.options );
			end
		end
		
	-- As we log out disable the plugin so that we can recreate a new frame next time we log in
	elseif ( event == "PLAYER_LOGOUT" ) then
		if ( addonData.ChatFrame ) then plugin.setEnabled(false); end
		
	end
	
	-- And at least for a while allow slash commands for testing purposes at least
	SLASH_nUI_InfoPanel_WhispersCmd1 = '/nuiwc';
	SlashCmdList['nUI_InfoPanel_WhispersCmd'] = nUI_InfoPanel_Whispers_SlashCommands;		                     
end

--[[ Register the events we want to watch ]]--
plugin:SetScript( "OnEvent", nUI_InfoPanel_Whispers_OnEvent );
plugin:RegisterEvent( "PLAYER_LOGIN" );
plugin:RegisterEvent( "UPDATE_CHAT_WINDOWS" );
plugin:RegisterEvent( "PLAYER_LOGOUT" );

